#!/usr/bin/env python

"""
discard_m.py
Accretion derived from discrad_mass.out.
Miroslav Broz (miroslav.broz@email.cz), May 29th 2020

"""

import sys
from math import sqrt, acos

gms = 39.47841760
M_S = 1.989e30
M_E = 5.972e24
fac = 1.0/gms*M_S/M_E

def orbel(x, y, z, vx, vy, vz, gmsum):

    r = sqrt(x*x + y*y + z*z)
    v2 = vx*vx + vy*vy + vz*vz
    energy = 0.5*v2 - gmsum/r

    hx = y*vz - z*vy
    hy = z*vx - x*vz
    hz = x*vy - y*vx
    h2 = hx*hx + hy*hy + hz*hz

    cos2i = hz*hz / h2
    inc = acos(sqrt(cos2i))

    if energy < 0.0:
        ialpha = -1
    else:
        ialpha = 1

    if ialpha == -1:
        a = -0.5*gmsum/energy  
        fac = 1.0 - h2/(gmsum*a)
        e = sqrt(fac)
    else:
        print "Error: not elliptic!"
        sys.exit(1)

    return a, e, inc

def read1(f):

    l = f.readline().split()
    id = int(l[1])
    m = float(l[2])
    x, y, z = map(lambda x: float(x), f.readline().split())
    vx, vy, vz = map(lambda x: float(x), f.readline().split())
    a, e, i = orbel(x, y, z, vx, vy, vz, gms+m)
    m = m*fac

    return id, m, a, e, i

def discard(f, tmax):

    ids = {}
    for i in xrange(0,29+1):
        ids[i] = [i]
    print "ids = ", ids

    f3 = open("initial.out", "r")
    ms = {}
    for line in f3.readlines():
        l = line.split()
        id_ = abs(int(l[0]))
        ms[id_] = float(l[8])
    f3.close()

    f2 = open("discard_a.out", "w")
    f2.write("# tmax = %.8e\n" % (tmax))
    f2.write("# no t [yr]          m1     m2     m [m_E]  a1     a2     a[au]   id1 id2  N1 N2   | ids1 || ids2 \n")

    k = 0
    while True:
       
        try: 
            l = f.readline().split()
            t = float(l[0])
        except (IOError,IndexError) as ex:
#            print ex
            break

        if t > tmax:
            break

        k += 1
        id1, m1, a1, e1, i1 = read1(f)
        id2, m2, a2, e2, i2 = read1(f)
        id_, m_, a_, e_, i_ = read1(f)

        n = len(ids)

        print ""
        print "t = ", t
        print "id1 = ", id1
        print "id2 = ", id2
        print "id_ = ", id_
        print "n = ", n
        print "ids = ", ids
        print ""

        f2.write("%2d %.10e  %.4f %.4f %.4f   %.4f %.4f %.4f  %2d  %2d  %2d  %2d   | " % (k, t, m1, m2, m_, a1, a2, a_, id1, id2, len(ids[id1]), len(ids[id2])))
        tmp = ids[id1][:]
        tmp.sort()
        for j in xrange(0,len(tmp)):
            f2.write(" %2d" % (tmp[j]))
        f2.write(" ||")
        tmp = ids[id2][:]
        tmp.sort()
        for j in xrange(0,len(tmp)):
            f2.write(" %2d" % (tmp[j]))
        f2.write("\n")

        if id_ == id1:
            ids[id1] += ids[id2]
            ids.pop(id2)
            for i in xrange(id2+1,n):
                ids[i-1] = ids[i]
            if id2 < n-1:
                ids.pop(n-1)
        else:
            print "Error: Merged particle does NOT have id = id1!"
            sys.exit(1)

        print "ids = ", ids

    print ""
    for i in xrange(2,len(ids)):
        f2.write("#  %.10e                                               %2d      %2d       | " % (t, i, len(ids[i])))
        tmp = ids[i][:]
        tmp.sort()
        for j in xrange(0,len(tmp)):
            f2.write(" %2d" % (tmp[j]))
        f2.write("\n")
        
    f.close()
    f2.close()

def main():
    if len(sys.argv) > 1:
        tmax = float(sys.argv[1])
    else:
        tmax = 3.0e7
        print "Usage: discard_a.py tmax"
    print "# tmax = ", tmax

    f = open("discard_mass.out", "r")
#    f = sys.stdin
    discard(f, tmax)

if __name__ == "__main__":
    main()


